#include <iostream>
#include <random>
#include <fstream>
#include <cstdlib>
#include <ctime> 
#include <cassert>
#include <unordered_set>
#include "graph.hpp"

using namespace std;

/******************/
/* SPECIAL GRAPHS */
/******************/

Graph clique(int n) {
  Graph g(n);
  for (int i = 0; i<n; i++){
    for (int j = i+1; j<n; j++){
      g.add_edge(i,j);
    }
  }
  return g;
}

Graph biclique(int n, int p) {
  Graph g(2*n);
  for (int i = 0; i<n; i++){
    for (int j = n; j<n+p; j++){
      g.add_edge(i,j);
    }
  }
}

/****************/
/* ALGO GLOUTON */
/****************/

// algo glouton
unordered_set<int> greedy_det(Graph& G) {
  unordered_set<int> IS = {}; 
  for (int i : G.vertices()) {
    bool tmp = true;
    for (int j : G.neighbors(i)) {
      if (IS.find(j) != IS.end())
	tmp = false;
    }
    if (tmp) IS.insert(i);
  }
  return IS;
}


/**************/
/* ALGO EXACT */
/**************/

// Fonction auxiliaire
unordered_set<int> exact_aux(Graph& g, vector<bool>& is_free, int i) {
  if (i == g.get_size()) {
    unordered_set<int> res = {};
    for (unsigned int j = 0; j < is_free.size(); j++) 
      if (is_free.at(j)) res.insert(j);
    return res;
  }
  if (!is_free.at(i)) return exact_aux(g, is_free, i+1);
  is_free.at(i) = false;
  unordered_set<int> case1 = exact_aux(g, is_free, i+1);
  is_free.at(i) = true;
  vector<bool> copy = is_free;
  for (int j : g.neighbors(i)) 
    is_free.at(j) = false;
  unordered_set<int> case2 = exact_aux(g, is_free, i+1);
  for (int j : g.neighbors(i)) 
    is_free.at(j) = copy.at(j);
  if (case1.size() > case2.size())
    return case1;
  else
    return case2;
}

// Algorithme exact
unordered_set<int> exact(Graph& g) {
  vector<bool> is_free(g.get_size(), true);
  return exact_aux(g, is_free, 0);
}
    
    

/********/
/* MAIN */
/********/

int main() {
  return 0;
}
